import '../../../appsetting.dart';

import '../entity/entuser.dart';
import '../entity/entuserweb.dart';
import '../entity/entrole.dart';
import '../entity/entroleweb.dart';
import '../entity/userfield.dart';
import '../entity/rolefield.dart';
import '../entity/fetchentityresult.dart';

import 'package:ingdao/ingdao.dart';
import 'package:ingdao/dataflow/field/dfbasefield.dart';
import 'package:ingdao/dataflow/control/dfquery.dart';
import 'package:ingdao/dataflow/control/dfquerypaging.dart';
import 'package:ingdao/dataflow/control/dfqueryorderby.dart';
import 'package:ingdao/dataflow/control/dfresponse.dart';
import 'package:ingdao/dataflow/control/dfquerycriteriagroup.dart';
import 'package:ingdao/dataflow/control/dfquerycriteria.dart';
import 'package:ingdao/dataflow/control/dfqueryentityjoin.dart';
import 'package:ingdao/dataflow/service/dfservice.dart';
import 'package:ingdao/dataflow/model/dfwebmodel.dart';
import 'package:ingdao/dataflow/repository/dfwebrepository.dart';
import 'package:ingdao/dataflow/model/dfdbmodel.dart';
import 'package:ingdao/dataflow/repository/dfdbrepository.dart';
import 'package:ingdao/dataflow/entity/dfbaseentity.dart';

class AppService extends DfService {
  DfFactory dfFactory = DfFactory.instance;

  @override
  startInit() async {
    AppSetting appSetting = AppSetting();

    //Web
    SslPinningCri sslPinningCri = SslPinningCri(shakeys: appSetting.sslPinningFingerPrints, shatype: SslPinningCriShaType.SHA256);
    DfWebModel modelWeb = DfWebModel(name: RepositoryType.web.name, 
                                    encrypt: appSetting.dataEncrypt, 
                                    entities: [RoleWeb(),UserWeb()], 
                                    serverUrl: appSetting.serverUrl, 
                                    commonHeaders: DfWebModel.formJsonHeader(authToken: appSetting.token), 
                                    sslPinningCri: sslPinningCri);

    DfWebRepository repWeb = DfWebRepository(model: modelWeb); 
    dfFactory.registerRepository(RepositoryType.web.name, repWeb);

    //Database
    DfDbModel modelDb = DfDbModel(name: RepositoryType.sqlitedb.name, 
                                  dbName: appSetting.dbName, 
                                  encrypt: appSetting.dataEncrypt, 
                                  entities: [Role(),User()]);
    DfDbRepository repDb = DfDbRepository(model: modelDb);
    dfFactory.registerRepository(RepositoryType.sqlitedb.name, repDb);

    //Activate
    //await dfFactory.activateRepository(RepositoryType.sqlitedb.name);
    await dfFactory.activateRepository(RepositoryType.web.name);

    await _createDftData();
  }

  _createDftData() async {
    DfBaseEntity entRole = dfFactory.currentRepository.model.entities[0];
    DfBaseEntity entUser = dfFactory.currentRepository.model.entities[1];

    FetchEntityResult<UserField> result = await fetchUser("", 1, 10);
    if (result.entities != null && result.entities.length > 0) {
      print("Default data there already.");
      return;
    }

    List<DfBaseEntity> roles = List<DfBaseEntity>();

    RoleField roleSuperAdmin = RoleField.createRoleFieldBasedOnRepo(dfFactory.currentRepository);
    roleSuperAdmin.fId.value = "superadmin";
    roleSuperAdmin.fName.value = "Super Admin";
    roles.add(roleSuperAdmin as DfBaseEntity);

    RoleField roleAdmin = RoleField.createRoleFieldBasedOnRepo(dfFactory.currentRepository);
    roleAdmin.fId.value = "admin";
    roleAdmin.fName.value = "Admin";
    roles.add(roleAdmin as DfBaseEntity);

    RoleField roleGeneraluser = RoleField.createRoleFieldBasedOnRepo(dfFactory.currentRepository);
    roleGeneraluser.fId.value = "generaluser";
    roleGeneraluser.fName.value = "General User";
    roles.add(roleGeneraluser as DfBaseEntity);

    List<DfQuery> roleInsertQueries = List<DfQuery>();
    roles.forEach((role) {
      DfQuery query = DfQuery(purpose: DfQueryPurpose.ADD, queryFields: role.toMap(), entityJoin: DfQueryEntityJoin(entities: [entRole])  );
      roleInsertQueries.add(query);
    });
    await dfFactory.save(roleInsertQueries);

    List<DfBaseEntity> users = List<DfBaseEntity>();
    for (int i = 1; i <= 1000; i++) {
      UserField user = UserField.createUserFieldBasedOnRepo(dfFactory.currentRepository);
      user.fId.value = "user$i";
      user.fName.value = "User $i";
      user.fEmail.value = "user$i@email.com";
      user.fPassword.value = "abc123";
      user.fRoleid.value = "generaluser";
      user.fPicture.value = "https://scontent.fkul14-1.fna.fbcdn.net/v/t1.0-9/16196015_10154888128487744_6901111466535510271_n.png?_nc_cat=103&ccb=2&_nc_sid=85a577&_nc_ohc=F5hFyf4b7r8AX9SkjRh&_nc_ht=scontent.fkul14-1.fna&oh=219538a2a70f159e41d9072537ce09ef&oe=5FBA9097";
      users.add(user as DfBaseEntity);
    }

    List<DfQuery> userInsertQueries = List<DfQuery>();
    users.forEach((user) {
      DfQuery query = DfQuery(purpose: DfQueryPurpose.ADD, queryFields: user.toMap(), entityJoin: DfQueryEntityJoin(entities: [entUser])  );
      userInsertQueries.add(query);
    });
    
    await dfFactory.save(userInsertQueries);
  }

  Future<int> fetchUserCount() async {
    DfBaseEntity entRole = dfFactory.currentRepository.model.entities[0];
    DfBaseEntity entUser = dfFactory.currentRepository.model.entities[1];
    DfQueryCriteriaGroup criteriaGroup = DfQueryCriteriaGroup(criterias: [_formUserRoleCriteria()]);
    DfQuery query = DfQuery(purpose: DfQueryPurpose.FETCH, 
                            queryFields: [entUser.getSelectQueryCountField()], 
                            entityJoin: DfQueryEntityJoin(entities: [entUser,entRole], 
                            joins: [JoinType.innerJoin]), 
                            criteriaGroup: criteriaGroup);

    DfResponse resp = await dfFactory.fetch(query);
    int count = 0;
    if (resp.error != null) {
      print("FetchUserCount Error: ${resp.error.message}");
    }
    else {
      resp.result[0].forEach((k,v) => count = v);
    }
    return count; 
  }

  DfQueryCriteria _formUserRoleCriteria() {
    RoleField role = RoleField.createRoleFieldBasedOnRepo(dfFactory.currentRepository);
    return DfQueryCriteria(field: role.fId, oper: OperType.eq, values: ['generaluser'] );
  }

  Future<FetchEntityResult<UserField>> fetchUser(String searchName, int pageNumber, int limit) async {
    DfBaseEntity entRole = dfFactory.currentRepository.model.entities[0];
    DfBaseEntity entUser = dfFactory.currentRepository.model.entities[1];
    UserField userField = UserField.createUserFieldBasedOnRepo(dfFactory.currentRepository);

    List<DfBaseField> selFields = [userField.fId, userField.fName, userField.fEmail, userField.fRoleName, userField.fPicture];

    List<DfQueryCriteria> criterias = [_formUserRoleCriteria()];
    if (searchName != null && searchName.length > 0) {
      criterias.add(DfQueryCriteria(field: userField.fName, oper: OperType.like, values: [searchName] ));
    }    

    DfQueryOrderBy orderBy =DfQueryOrderBy(field: userField.fName, orderByType: OrderByType.asc);
    Paging paging = Paging(pageNumber: pageNumber, perPageCount: limit);

    DfQuery query = DfQuery(purpose: DfQueryPurpose.FETCH, 
                            queryFields: selFields, 
                            entityJoin: DfQueryEntityJoin(entities: [entUser,entRole], 
                            joins: [JoinType.innerJoin]), 
                            criteriaGroup: DfQueryCriteriaGroup(criterias: criterias), 
                            orderBy: [orderBy], 
                            paging: paging);

    List<UserField> users = List<UserField>();
    DfResponse resp = await dfFactory.fetch(query);
    if (resp.error != null) {
      print("FetchUser Error: ${resp.error.message}");
    }
    else {
      List<Map<String,dynamic>> resultSet = resp.result;
      resultSet.forEach((result) {
        DfBaseEntity newUser = entUser.newInstance();
        print("Final result $result");
        newUser.fromMap(result);
        UserField uf = newUser as UserField;
        print("Final user role name ${uf.fRoleName.value}");
        users.add(uf);
      });
    }
    return FetchEntityResult<UserField>(entities: users, error: resp.error != null ? resp.error.message : null);
  }

}