import 'package:meta/meta.dart';
import 'dfbaseentity.dart';
import '../field/dfdbfield.dart';
import '../field/dfbasefield.dart';
import '../control/dfquerycriteria.dart';
import '../control/dfquerycriteriagroup.dart';

abstract class DfDbEntity extends DfBaseEntity<DfDbField> {

  DfDbEntity({@required String name}) : super(name: name);

   List<DfDbField> getJoinKeyFields() {
    return fields().where((f) => f.joinKey).toList();
  }

  List<DfDbField> getPrimaryKeyFields() {
    return fields().where((f) => f.primaryKey).toList();
  }

  DfQueryCriteriaGroup getPrimaryKeyDaoCriteriaList() {
    List<DfQueryCriteria> criGroupArr = List<DfQueryCriteria>();
    getPrimaryKeyFields().forEach((field) {
      criGroupArr.add(DfQueryCriteria(field: field, oper: OperType.eq, values: [field.value]));
    });
    return DfQueryCriteriaGroup(criterias: criGroupArr);

  }

  String toPrimaryKeyStr() {
    String result = "";
    getPrimaryKeyFields().forEach((item) {
      if (result.length > 0) {
        result = "$result, ";        
      }
      String fName = item.name;
      result = "$result$fName";
    });
    if (result.length > 0) {
      return ", PRIMARY KEY ($result)";
    }
    return '';
  }

  String genAlterTblAddFieldStr(DfDbField field) {
    return "ALTER TABLE $name ADD ${field.toCreateFieldStr()}";
  }

  String toCreateEntityStr() {
    String result = "";
    fields().forEach((field) {
      if (field.displayOnly == null || !field.displayOnly) {
        if (result.length > 0) {
          result = "$result, ";
        }
        String fieldCreation = field.toCreateFieldStr();
        result = "$result$fieldCreation";
      }
    });
    String priKeyStr = toPrimaryKeyStr();
    return "CREATE TABLE $name($result$priKeyStr)";
  }

  DfBaseField getSelectQueryCountField() {
    return DfDbField(entity: this, name: "count(*)", primaryKey: false, joinKey: false, type: FieldType.integer);
  }

}