import 'package:meta/meta.dart';
import 'dfbaseentity.dart';
import '../field/dfwebfield.dart';
import '../field/dfbasefield.dart';
import '../control/dfquery.dart';

enum HttpMethodType {
  GET,
  POST,
  PUT,
  PATCH,
  DELETE
}

extension FieldTypeExtension on HttpMethodType {

  String get name {
    switch (this) {
      case HttpMethodType.GET:
        return 'GET';
      case HttpMethodType.POST:
        return 'POST';
     case HttpMethodType.PUT:
        return 'PUT';
     case HttpMethodType.PATCH:
        return 'PATCH';
     case HttpMethodType.DELETE:
        return 'DELETE';        
      default:
        return null;
    }
  }

  String desp() {
    return this.name;
  }
}

abstract class DfWebEntity extends DfBaseEntity<DfWebField> {

  Map<String,String> headers;

  DfWebEntity({@required String contextPath, this.headers}) : super(name: contextPath);

  String formUrl(DfQuery query) {
    DfWebReqParam params = convertToWebReqParam(query);  
    String url = name;
    if (params.pathVariables != null) {
      params.pathVariables.forEach((pVar) => url = "$url/$pVar");
    }
    return url;
  }

  DfBaseField getSelectQueryCountField() {
    return DfWebField(entity: this, name: "count(*)");
  }

  DfWebReqParam convertToWebReqParam(DfQuery query);  
}

class DfWebReqParam {
  List<String> pathVariables;
  Map<String,String> reqParams;
  Map<String,dynamic> reqBody;

  DfWebReqParam({this.pathVariables, this.reqParams, this.reqBody}) {
    if (pathVariables == null) {
      pathVariables = List<String>();
    }
    if (reqParams == null) {
      reqParams = Map<String,String>();
    }
    if (reqBody == null) {
      reqBody = Map<String,dynamic>();
    }
  }
}